@@ -43,18 +43,38 @@ pub enum MIDIEvent {
 	NoteOff(u8, u8, u8),
 	AfterTouch(u8, u8, u8),
 
-	BankSelect(u8, u16),
-	ModulationWheel(u8, u16),
-	BreathController(u8, u16),
-	FootPedal(u8, u16),
-	PortamentoTime(u8, u16),
-	DataEntrySlider(u8, u16),
-	Volume(u8, u16),
-	Balance(u8, u16),
-	Pan(u8, u16),
-	Expression(u8, u16),
-	EffectControl(u8, u8, u16),
-	Slider(u8, u8, u8),
+    BankSelect(u8, u8),
+    BankSelectLSB(u8, u8),
+    ModulationWheel(u8, u8),
+    ModulationWheelLSB(u8, u8),
+    BreathController(u8, u8),
+    BreathControllerLSB(u8, u8),
+    FootPedal(u8, u8),
+    FootPedalLSB(u8, u8),
+    PortamentoTime(u8, u8),
+    PortamentoTimeLSB(u8, u8),
+    DataEntry(u8, u8),
+    DataEntryLSB(u8, u8),
+    Volume(u8, u8),
+    VolumeLSB(u8, u8),
+    Balance(u8, u8),
+    BalanceLSB(u8, u8),
+    Pan(u8, u8),
+    PanLSB(u8, u8),
+    Expression(u8, u8),
+    ExpressionLSB(u8, u8),
+    EffectControl1(u8, u8),
+    EffectControl1LSB(u8, u8),
+    EffectControl2(u8, u8),
+    EffectControl2LSB(u8, u8),
+    GeneralPurpose1(u8, u8),
+    GeneralPurpose1LSB(u8, u8),
+    GeneralPurpose2(u8, u8),
+    GeneralPurpose2LSB(u8, u8),
+    GeneralPurpose3(u8, u8),
+    GeneralPurpose3LSB(u8, u8),
+    GeneralPurpose4(u8, u8),
+    GeneralPurpose4LSB(u8, u8),
 	HoldPedal(u8, u8),
 	Portamento(u8, u8),
 	Sustenuto(u8, u8),
@@ -66,21 +86,28 @@ pub enum MIDIEvent {
 	SoundReleaseTime(u8, u8),
 	SoundAttack(u8, u8),
 	SoundBrightness(u8, u8),
-	SoundControl(u8, u8, u8),
-	GeneralButtonOn(u8, u8),
-	GeneralButtonOff(u8, u8),
+	SoundControl1(u8, u8),
+	SoundControl2(u8, u8),
+	SoundControl3(u8, u8),
+	SoundControl4(u8, u8),
+	SoundControl5(u8, u8),
+    GeneralPurpose5(u8, u8),
+    GeneralPurpose6(u8, u8),
+    GeneralPurpose7(u8, u8),
+    GeneralPurpose8(u8, u8),
 	EffectsLevel(u8, u8),
 	TremuloLevel(u8, u8),
 	ChorusLevel(u8, u8),
 	CelesteLevel(u8, u8),
 	PhaserLevel(u8, u8),
-	DataButtonIncrement(u8),
-	DataButtonDecrement(u8),
-	RegisteredParameterNumber(u8, u16),
-	NonRegisteredParameterNumber(u8, u16),
+	DataIncrement(u8),
+	DataDecrement(u8),
+    RegisteredParameterNumber(u8, u8),
+    RegisteredParameterNumberLSB(u8, u8),
+    NonRegisteredParameterNumber(u8, u8),
+    NonRegisteredParameterNumberLSB(u8, u8),
 	AllControllersOff(u8),
-	LocalKeyboardEnable(u8),
-	LocalKeyboardDisable(u8),
+	LocalControl(u8, u8),
 	AllNotesOff(u8),
 	AllSoundOff(u8),
 	OmniOff(u8),
@@ -88,6 +115,7 @@ pub enum MIDIEvent {
 	MonophonicOperation(u8, u8),
 	PolyphonicOperation(u8),
 	ControlChange(u8, u8, u8),
+
 	ProgramChange(u8, u8),
 	ChannelPressure(u8, u8),
 	PitchWheelChange(u8, f64),
@@ -96,6 +124,7 @@ pub enum MIDIEvent {
 	SongPositionPointer(u16),
 	SongSelect(u8),
     TimeCode(f32, u8, u8, u8, u8),
+
 	EndOfTrack,
     TuneRequest,
     MIDIClock,
@@ -262,100 +291,123 @@ impl MIDIBytes for MIDIEvent {
             }
 
             MIDIEvent::BankSelect(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x00, 0x20
-                )
+                vec![ 0xB0 | *channel, 0x00, *value ]
+            }
+            MIDIEvent::BankSelectLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x20, *value ]
             }
 
             MIDIEvent::ModulationWheel(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x01, 0x21
-                )
+                vec![ 0xB0 | *channel, 0x01, *value ]
+            }
+            MIDIEvent::ModulationWheelLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x21, *value ]
             }
 
             MIDIEvent::BreathController(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x02, 0x22
-                )
+                vec![ 0xB0 | *channel, 0x02, *value ]
+            }
+            MIDIEvent::BreathControllerLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x22, *value ]
             }
 
             MIDIEvent::FootPedal(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x04, 0x24
-                )
+                vec![ 0xB0 | *channel, 0x04, *value ]
+            }
+            MIDIEvent::FootPedalLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x24, *value ]
             }
 
             MIDIEvent::PortamentoTime(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x05, 0x25
-                )
+                vec![ 0xB0 | *channel, 0x05, *value ]
+            }
+            MIDIEvent::PortamentoTimeLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x25, *value ]
             }
 
-            MIDIEvent::DataEntrySlider(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x06, 0x26
-                )
+            MIDIEvent::DataEntry(channel, value) => {
+                vec![ 0xB0 | *channel, 0x06, *value ]
+            }
+            MIDIEvent::DataEntryLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x26, *value ]
             }
 
             MIDIEvent::Volume(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x07, 0x27
-                )
+                vec![ 0xB0 | *channel, 0x07, *value ]
+            }
+            MIDIEvent::VolumeLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x27, *value ]
             }
 
             MIDIEvent::Balance(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x08, 0x28
-                )
+                vec![ 0xB0 | *channel, 0x08, *value ]
+            }
+            MIDIEvent::BalanceLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x28, *value ]
             }
 
-
             MIDIEvent::Pan(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x0A, 0x2A
-                )
+                vec![ 0xB0 | *channel, 0x0A, *value ]
+            }
+            MIDIEvent::PanLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x2A, *value ]
             }
 
             MIDIEvent::Expression(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x0B, 0x2B
-                )
+                vec![ 0xB0 | *channel, 0x0B, *value ]
+            }
+            MIDIEvent::ExpressionLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x2B, *value ]
             }
 
-            MIDIEvent::EffectControl(channel, which, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x0C + *which, 0x2C + *which
-                )
+            MIDIEvent::EffectControl1(channel, value) => {
+                vec![ 0xB0 | *channel, 0x0C, *value ]
+            }
+            MIDIEvent::EffectControl1LSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x2C, *value ]
+            }
+            MIDIEvent::EffectControl2(channel, value) => {
+                vec![ 0xB0 | *channel, 0x0D, *value ]
+            }
+            MIDIEvent::EffectControl2LSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x2D, *value ]
             }
 
-            MIDIEvent::Slider(channel, which, value) => {
-                vec![
-                    0xB | *channel,
-                    0x10 + *which as u8,
-                    *value
-                ]
+            MIDIEvent::GeneralPurpose1(channel, value) => {
+                vec![ 0xB0 | *channel, 0x10, *value ]
+            }
+            MIDIEvent::GeneralPurpose1LSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x30, *value ]
+            }
+            MIDIEvent::GeneralPurpose2(channel, value) => {
+                vec![ 0xB0 | *channel, 0x11, *value ]
+            }
+            MIDIEvent::GeneralPurpose2LSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x31, *value ]
+            }
+            MIDIEvent::GeneralPurpose3(channel, value) => {
+                vec![ 0xB0 | *channel, 0x12, *value ]
+            }
+            MIDIEvent::GeneralPurpose3LSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x32, *value ]
+            }
+            MIDIEvent::GeneralPurpose4(channel, value) => {
+                vec![ 0xB0 | *channel, 0x13, *value ]
+            }
+            MIDIEvent::GeneralPurpose4LSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x33, *value ]
+            }
+            MIDIEvent::GeneralPurpose5(channel, value) => {
+                vec![ 0xB0 | *channel, 0x50, *value]
+            }
+            MIDIEvent::GeneralPurpose6(channel, value) => {
+                vec![ 0xB0 | *channel, 0x51, *value]
+            }
+            MIDIEvent::GeneralPurpose7(channel, value) => {
+                vec![ 0xB0 | *channel, 0x52, *value]
+            }
+            MIDIEvent::GeneralPurpose8(channel, value) => {
+                vec![ 0xB0 | *channel, 0x53, *value]
             }
 
             MIDIEvent::HoldPedal(channel, value) => {
@@ -368,7 +420,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::Portamento(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x41,
                     *value
                 ]
@@ -376,7 +428,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::Sustenuto(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x42,
                     *value
                 ]
@@ -384,7 +436,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::SoftPedal(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x43,
                     *value
                 ]
@@ -392,7 +444,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::Legato(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x44,
                     *value
                 ]
@@ -400,7 +452,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::Hold2Pedal(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x45,
                     *value
                 ]
@@ -408,7 +460,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::SoundVariation(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x46,
                     *value
                 ]
@@ -416,7 +468,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::SoundTimbre(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x47,
                     *value
                 ]
@@ -424,7 +476,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::SoundReleaseTime(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x48,
                     *value
                 ]
@@ -432,7 +484,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::SoundAttack(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x49,
                     *value
                 ]
@@ -440,39 +492,31 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::SoundBrightness(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x4A,
                     *value
                 ]
             }
 
-            MIDIEvent::SoundControl(channel, which, value) => {
-                vec![
-                    0xb0 | *channel,
-                    0x4B + *which,
-                    *value
-                ]
+            MIDIEvent::SoundControl1(channel, value) => {
+                vec![ 0xB0 | *channel, 0x4B, *value ]
             }
-
-            MIDIEvent::GeneralButtonOn(channel, which) => {
-                vec![
-                    0xB0 | *channel,
-                    0x50 + *which,
-                    0b01000000
-                ]
+            MIDIEvent::SoundControl2(channel, value) => {
+                vec![ 0xB0 | *channel, 0x4C, *value ]
             }
-
-            MIDIEvent::GeneralButtonOff(channel, which) => {
-                vec![
-                    0xB0 | *channel,
-                    0x50 + *which,
-                    0b00000000
-                ]
+            MIDIEvent::SoundControl3(channel, value) => {
+                vec![ 0xB0 | *channel, 0x4D, *value ]
+            }
+            MIDIEvent::SoundControl4(channel, value) => {
+                vec![ 0xB0 | *channel, 0x4E, *value ]
+            }
+            MIDIEvent::SoundControl5(channel, value) => {
+                vec![ 0xB0 | *channel, 0x4F, *value ]
             }
 
             MIDIEvent::EffectsLevel(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x5B,
                     *value
                 ]
@@ -480,7 +524,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::TremuloLevel(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x5C,
                     *value
                 ]
@@ -488,7 +532,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::ChorusLevel(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x5D,
                     *value
                 ]
@@ -496,7 +540,7 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::CelesteLevel(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x5E,
                     *value
                 ]
@@ -504,35 +548,33 @@ impl MIDIBytes for MIDIEvent {
 
             MIDIEvent::PhaserLevel(channel, value) => {
                 vec![
-                    0xb0 | *channel,
+                    0xB0 | *channel,
                     0x5F,
                     *value
                 ]
             }
 
-            MIDIEvent::DataButtonIncrement(channel) => {
+            MIDIEvent::DataIncrement(channel) => {
                 vec![0xB0 | *channel, 0x60, 0x00]
             }
 
-            MIDIEvent::DataButtonDecrement(channel) => {
+            MIDIEvent::DataDecrement(channel) => {
                 vec![0xB0 | *channel, 0x61, 0x00]
             }
 
+            MIDIEvent::NonRegisteredParameterNumber(channel, value) => {
+                vec![ 0xB0 | *channel, 0x63, *value ]
+            }
+            MIDIEvent::NonRegisteredParameterNumberLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x62, *value ]
+            }
             MIDIEvent::RegisteredParameterNumber(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x65, 0x64
-                )
+                vec![ 0xB0 | *channel, 0x65, *value ]
+            }
+            MIDIEvent::RegisteredParameterNumberLSB(channel, value) => {
+                vec![ 0xB0 | *channel, 0x64, *value ]
             }
 
-            MIDIEvent::NonRegisteredParameterNumber(channel, value) => {
-                gen_coarse_fine_bytes(
-                    *channel,
-                    *value,
-                    0x65, 0x64
-                )
-            }
 
             MIDIEvent::AllControllersOff(channel) => {
                 vec![
@@ -541,18 +583,8 @@ impl MIDIBytes for MIDIEvent {
                 ]
             }
 
-            MIDIEvent::LocalKeyboardEnable(channel) => {
-                vec![
-                    0xB0 | *channel,
-                    0x7A, 0x40
-                ]
-            }
-
-            MIDIEvent::LocalKeyboardDisable(channel) => {
-                vec![
-                    0xB0 | *channel,
-                    0x7A, 0x00
-                ]
+            MIDIEvent::LocalControl(channel, value) => {
+                vec![ 0xB0 | *channel, 0x7A, *value ]
             }
 
             MIDIEvent::AllNotesOff(channel) => {
@@ -744,8 +776,233 @@ impl MIDIBytes for MIDIEvent {
                         channel = leadbyte & 0x0F;
                         let controller = bytes.remove(0);
                         let value = bytes.remove(0);
-                        let event = MIDIEvent::ControlChange(channel, controller, value);
-                        output = Ok(event);
+                        output = match controller {
+                            0x00 => {
+                                Ok(MIDIEvent::BankSelect(channel, value))
+                            }
+                            0x20 => {
+                                Ok(MIDIEvent::BankSelectLSB(channel, value))
+                            }
+                            0x01 => {
+                                Ok(MIDIEvent::ModulationWheel(channel, value))
+                            }
+                            0x21 => {
+                                Ok(MIDIEvent::ModulationWheelLSB(channel, value))
+                            }
+                            0x02 => {
+                                Ok(MIDIEvent::BreathController(channel, value))
+                            }
+                            0x22 => {
+                                Ok(MIDIEvent::BreathControllerLSB(channel, value))
+                            }
+                            0x04 => {
+                                Ok(MIDIEvent::FootPedal(channel, value))
+                            }
+                            0x24 => {
+                                Ok(MIDIEvent::FootPedalLSB(channel, value))
+                            }
+                            0x05 => {
+                                Ok(MIDIEvent::PortamentoTime(channel, value))
+                            }
+                            0x25 => {
+                                Ok(MIDIEvent::PortamentoTimeLSB(channel, value))
+                            }
+                            0x06 => {
+                                Ok(MIDIEvent::DataEntry(channel, value))
+                            }
+                            0x26 => {
+                                Ok(MIDIEvent::DataEntryLSB(channel, value))
+                            }
+                            0x07 => {
+                                Ok(MIDIEvent::Volume(channel, value))
+                            }
+                            0x27 => {
+                                Ok(MIDIEvent::VolumeLSB(channel, value))
+                            }
+                            0x08 => {
+                                Ok(MIDIEvent::Balance(channel, value))
+                            }
+                            0x28 => {
+                                Ok(MIDIEvent::BalanceLSB(channel, value))
+                            }
+                            0x0A => {
+                                Ok(MIDIEvent::Pan(channel, value))
+                            }
+                            0x2A => {
+                                Ok(MIDIEvent::PanLSB(channel, value))
+                            }
+                            0x0B => {
+                                Ok(MIDIEvent::Expression(channel, value))
+                            }
+                            0x2B => {
+                                Ok(MIDIEvent::ExpressionLSB(channel, value))
+                            }
+                            0x0C => {
+                                Ok(MIDIEvent::EffectControl1(channel, value))
+                            }
+                            0x2C => {
+                                Ok(MIDIEvent::EffectControl1LSB(channel, value))
+                            }
+                            0x0D => {
+                                Ok(MIDIEvent::EffectControl2(channel, value))
+                            }
+                            0x2D => {
+                                Ok(MIDIEvent::EffectControl2LSB(channel, value))
+                            }
+
+                            0x10 => {
+                                Ok(MIDIEvent::GeneralPurpose1(channel, value))
+                            }
+                            0x30 => {
+                                Ok(MIDIEvent::GeneralPurpose1LSB(channel, value))
+                            }
+                            0x11 => {
+                                Ok(MIDIEvent::GeneralPurpose2(channel, value))
+                            }
+                            0x31 => {
+                                Ok(MIDIEvent::GeneralPurpose2LSB(channel, value))
+                            }
+                            0x12 => {
+                                Ok(MIDIEvent::GeneralPurpose3(channel, value))
+                            }
+                            0x32 => {
+                                Ok(MIDIEvent::GeneralPurpose3LSB(channel, value))
+                            }
+                            0x13 => {
+                                Ok(MIDIEvent::GeneralPurpose4(channel, value))
+                            }
+                            0x33 => {
+                                Ok(MIDIEvent::GeneralPurpose4LSB(channel, value))
+                            }
+                            0x40 => {
+                                Ok(MIDIEvent::HoldPedal(channel, value))
+                            }
+                            0x41 => {
+                                Ok(MIDIEvent::Portamento(channel, value))
+                            }
+                            0x42 => {
+                                Ok(MIDIEvent::Sustenuto(channel, value))
+                            }
+                            0x43 => {
+                                Ok(MIDIEvent::SoftPedal(channel, value))
+                            }
+                            0x44 => {
+                                Ok(MIDIEvent::Legato(channel, value))
+                            }
+                            0x45 => {
+                                Ok(MIDIEvent::Hold2Pedal(channel, value))
+                            }
+                            0x46 => {
+                                Ok(MIDIEvent::SoundVariation(channel, value))
+                            }
+                            0x47 => {
+                                Ok(MIDIEvent::SoundTimbre(channel, value))
+                            }
+                            0x48 => {
+                                Ok(MIDIEvent::SoundReleaseTime(channel, value))
+                            }
+                            0x49 => {
+                                Ok(MIDIEvent::SoundAttack(channel, value))
+                            }
+                            0x4A => {
+                                Ok(MIDIEvent::SoundBrightness(channel, value))
+                            }
+                            0x4B => {
+                                Ok(MIDIEvent::SoundControl1(channel, value))
+                            }
+                            0x4C => {
+                                Ok(MIDIEvent::SoundControl2(channel, value))
+                            }
+                            0x4D => {
+                                Ok(MIDIEvent::SoundControl3(channel, value))
+                            }
+                            0x4E => {
+                                Ok(MIDIEvent::SoundControl4(channel, value))
+                            }
+                            0x4F => {
+                                Ok(MIDIEvent::SoundControl5(channel, value))
+                            }
+                            0x50 => {
+                                Ok(MIDIEvent::GeneralPurpose5(channel, value))
+                            }
+                            0x51 => {
+                                Ok(MIDIEvent::GeneralPurpose6(channel, value))
+                            }
+                            0x52 => {
+                                Ok(MIDIEvent::GeneralPurpose7(channel, value))
+                            }
+                            0x53 => {
+                                Ok(MIDIEvent::GeneralPurpose8(channel, value))
+                            }
+
+                            0x5B => {
+                                Ok(MIDIEvent::EffectsLevel(channel, value))
+                            }
+
+                            0x5C => {
+                                Ok(MIDIEvent::TremuloLevel(channel, value))
+                            }
+
+                            0x5D => {
+                                Ok(MIDIEvent::ChorusLevel(channel, value))
+                            }
+                            0x5E => {
+                                Ok(MIDIEvent::CelesteLevel(channel, value))
+                            }
+
+                            0x5F => {
+                                Ok(MIDIEvent::PhaserLevel(channel, value))
+                            }
+
+                            0x60 => {
+                                Ok(MIDIEvent::DataIncrement(channel))
+                            }
+
+                            0x61 => {
+                                Ok(MIDIEvent::DataDecrement(channel))
+                            }
+                            0x62 => {
+                                Ok(MIDIEvent::NonRegisteredParameterNumberLSB(channel, value))
+                            }
+
+                            0x63 => {
+                                Ok(MIDIEvent::NonRegisteredParameterNumber(channel, value))
+                            }
+
+                            0x64 => {
+                                Ok(MIDIEvent::RegisteredParameterNumberLSB(channel, value))
+                            }
+                            0x65 => {
+                                Ok(MIDIEvent::RegisteredParameterNumber(channel, value))
+                            }
+                            0x78 => {
+                                Ok(MIDIEvent::AllSoundOff(channel))
+                            }
+                            0x79 => {
+                                Ok(MIDIEvent::AllControllersOff(channel))
+                            }
+                            0x7A => {
+                                Ok(MIDIEvent::LocalControl(channel, value))
+                            }
+                            0x7B => {
+                                Ok(MIDIEvent::AllNotesOff(channel))
+                            }
+                            0x7C => {
+                                Ok(MIDIEvent::OmniOff(channel))
+                            }
+                            0x7D => {
+                                Ok(MIDIEvent::OmniOn(channel))
+                            }
+                            0xFE => {
+                                Ok(MIDIEvent::MonophonicOperation(channel, value))
+                            }
+                            0xFF => {
+                                Ok(MIDIEvent::PolyphonicOperation(channel))
+                            }
+                            _ => {
+                                Ok(MIDIEvent::ControlChange(channel, controller, value))
+                            }
+                        }
                     }
                     0xC => {
                         channel = leadbyte & 0x0F;
@@ -803,16 +1060,6 @@ impl MIDIBytes for MIDIEvent {
                 output = Ok(event);
             }
 
-            0xF1 | 0xF6 | 0xF8 | 0xFA | 0xFB | 0xFC | 0xFE => {
-                // Do Nothing. These are system-realtime and shouldn't be in a file.
-            }
-
-            0xF7 => {
-                varlength = get_variable_length_number(bytes);
-                n = dequeue_n(bytes, varlength as usize);
-                // TODO ADD EVENT
-            }
-
             0xFF => {
                 let meta_byte = bytes.remove(0); // Meta Type
                 varlength = get_variable_length_number(bytes);
@@ -940,6 +1187,10 @@ impl MIDIBytes for MIDIEvent {
                 }
             }
 
+            0xF1 | 0xF6 | 0xF8 | 0xFA | 0xFB | 0xFC | 0xFE | 0xF7 => {
+                // Do Nothing. These are system-realtime and shouldn't be in a file.
+            }
+
             0xF4 | 0xF5 | 0xF9 | 0xFD => {
                 // Undefined Behaviour
             }
@@ -1430,27 +1681,6 @@ fn build_pitch_wheel_change(channel: u8, lsb: u8, msb: u8) -> MIDIEvent {
     MIDIEvent::PitchWheelChange(channel, new_value)
 }
 
-fn gen_coarse_fine_bytes(channel: u8, value: u16, coarse_offset: u8, fine_offset: u8) -> Vec<u8> {
-    let mut output: Vec<u8> = Vec::new();
-
-    // MSB
-    if value > 0x7F {
-        output.push(0xB | channel);
-        output.push(coarse_offset);
-        output.push((value >> 7) as u8);
-    }
-
-    // LSB
-    if value & 0x7F != 0 {
-        output.push(0xB | channel);
-        output.push(fine_offset);
-        output.push((value & 0x7F) as u8);
-    }
-
-    output
-}
-
-
 fn get_mi_sf(chord_name: &str) -> (u8, i8) {
     match chord_name {
         "A" => (0, 3),
